"""
Version information for EGRC Core.
"""

__version__ = "1.1.0"
__version_info__ = (1, 1, 0)
__author__ = "EGRC Team"
__email__ = "team@egrc.com"
__description__ = "EGRC Core - Shared functionality and GraphQL support for Enterprise Governance, Risk, and Compliance platform"
__url__ = "https://github.com/egrc/egrc-core"
__license__ = "MIT"
